@@ -22,10 +22,10 @@
 #  <http://www.gnu.org/licenses/>.
 
 import textwrap
-import unittest
 from io import StringIO
 from os.path import abspath, dirname, join
 from subprocess import run
+from typing import List
 
 from mock import patch
 
@@ -48,22 +48,34 @@ from ansi2html.util import read_to_unicode
 _here = dirname(abspath(__file__))
 
 
-class TestAnsi2HTML(unittest.TestCase):
+class TestAnsi2HTML:
     maxDiff = None
 
-    def test_linkify(self):
+    def test_linkify(self) -> None:
         ansi = "http://threebean.org"
         target = '<a href="http://threebean.org">http://threebean.org</a>'
         html = Ansi2HTMLConverter(linkify=True).convert(ansi)
         assert target in html
 
-    def test_not_linkify(self):
+    def test_not_linkify(self) -> None:
         ansi = "http://threebean.org"
         target = '<a href="http://threebean.org">http://threebean.org</a>'
         html = Ansi2HTMLConverter().convert(ansi)
         assert target not in html
 
-    def test_conversion(self):
+    def test_osc_link(self) -> None:
+        ansi = "[\x1b[01;35m\x1b[K\x1b]8;;https://gcc.gnu.org/onlinedocs/gcc/Warning-Options.html#index-Wtype-limits\x07-Wtype-limits\x1b]8;;\x07\x1b[m\x1b[K]\n"
+        target = '[<span class="ansi1 ansi35"><a href="https://gcc.gnu.org/onlinedocs/gcc/Warning-Options.html#index-Wtype-limits">-Wtype-limits</a></span>]'
+        html = Ansi2HTMLConverter().convert(ansi)
+        assert target in html
+
+    def test_osc_link_latex(self) -> None:
+        ansi = "[\x1b[01;35m\x1b[K\x1b]8;;https://gcc.gnu.org/onlinedocs/gcc/Warning-Options.html#index-Wtype-limits\x07-Wtype-limits\x1b]8;;\x07\x1b[m\x1b[K]\n"
+        target = "[\\textcolor{ansi1 ansi35}{\\href{https://gcc.gnu.org/onlinedocs/gcc/Warning-Options.html#index-Wtype-limits}{-Wtype-limits}}]"
+        html = Ansi2HTMLConverter(latex=True).convert(ansi)
+        assert target in html
+
+    def test_conversion(self) -> None:
         for input_filename, expected_output_filename in (
             ("ansicolor.txt", "ansicolor.html"),
             ("ansicolor_eix.txt", "ansicolor_eix.html"),
@@ -82,18 +94,20 @@ class TestAnsi2HTML(unittest.TestCase):
             if html and html[-1] == "":
                 html = html[:-1]
 
-            self.assertEqual("\n".join(html), "\n".join(expected_data))
+            assert "\n".join(html) == "\n".join(expected_data)
 
     @patch("sys.argv", new_callable=lambda: ["ansi2html"])
     @patch("sys.stdout", new_callable=StringIO)
-    def test_conversion_as_command(self, mock_stdout, mock_argv):
+    def test_conversion_as_command(
+        self, mock_stdout: StringIO, mock_argv: List[str]
+    ) -> None:
         with open(join(_here, "ansicolor.txt"), "rb") as input:
             test_data = "".join(read_to_unicode(input))
 
         with open(join(_here, "ansicolor.html"), "rb") as output:
             expected_data = "".join(read_to_unicode(output))
 
-        def f():
+        def f() -> StringIO:
             return StringIO(test_data)
 
         with patch("sys.stdin", new_callable=f):
@@ -101,10 +115,10 @@ class TestAnsi2HTML(unittest.TestCase):
 
         html = mock_stdout.getvalue()
 
-        self.assertEqual(html, expected_data)
+        assert html == expected_data
 
-    def test_unicode(self):
-        """ Ensure that the converter returns unicode(py2)/str(py3) objs. """
+    def test_unicode(self) -> None:
+        """Ensure that the converter returns unicode(py2)/str(py3) objs."""
 
         with open(join(_here, "ansicolor.txt"), "rb") as input:
             test_data = "".join(read_to_unicode(input))
@@ -116,7 +130,9 @@ class TestAnsi2HTML(unittest.TestCase):
 
     @patch("sys.argv", new_callable=lambda: ["ansi2html", "--inline"])
     @patch("sys.stdout", new_callable=StringIO)
-    def test_inline_as_command(self, mock_stdout, mock_argv):
+    def test_inline_as_command(
+        self, mock_stdout: StringIO, mock_argv: List[str]
+    ) -> None:
         test_input = textwrap.dedent(
             """
         this is
@@ -132,16 +148,15 @@ class TestAnsi2HTML(unittest.TestCase):
 
     @patch("sys.argv", new_callable=lambda: ["ansi2html", "--partial"])
     @patch("sys.stdout", new_callable=StringIO)
-    def test_partial_as_command(self, mock_stdout, mock_argv):
+    def test_partial_as_command(
+        self, mock_stdout: StringIO, mock_argv: List[str]
+    ) -> None:
         rainbow = "\x1b[1m\x1b[40m\x1b[31mr\x1b[32ma\x1b[33mi\x1b[34mn\x1b[35mb\x1b[36mo\x1b[37mw\x1b[0m\n"
         with patch("sys.stdin", new_callable=lambda: StringIO(rainbow)):
             main()
 
         html = mock_stdout.getvalue().strip()
 
-        if hasattr(html, "decode"):
-            html = html.decode("utf-8")
-
         expected = (
             '<span class="ansi1"></span>'
             + '<span class="ansi1 ansi40"></span>'
@@ -155,9 +170,25 @@ class TestAnsi2HTML(unittest.TestCase):
         )
         assert isinstance(html, str)
         assert isinstance(expected, str)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_partial(self):
+    @patch("sys.argv", new_callable=lambda: ["ansi2html", "--headers"])
+    @patch("sys.stdout", new_callable=StringIO)
+    def test_headers_as_command(
+        self, mock_stdout: StringIO, mock_argv: List[str]
+    ) -> None:
+
+        main()
+
+        with open(join(_here, "produce_headers.txt"), "rb") as produce_headers:
+            expected = read_to_unicode(produce_headers)
+
+        html = mock_stdout.getvalue()
+
+        assert isinstance(html, str)
+        assert html == "".join(expected)
+
+    def test_partial(self) -> None:
         rainbow = "\x1b[1m\x1b[40m\x1b[31mr\x1b[32ma\x1b[33mi\x1b[34mn\x1b[35mb\x1b[36mo\x1b[37mw\x1b[0m\n"
 
         html = Ansi2HTMLConverter().convert(rainbow, full=False).strip()
@@ -172,9 +203,9 @@ class TestAnsi2HTML(unittest.TestCase):
             + '<span class="ansi1 ansi36 ansi40">o</span>'
             + '<span class="ansi1 ansi37 ansi40">w</span>'
         )
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_inline(self):
+    def test_inline(self) -> None:
 
         rainbow = "\x1b[1m\x1b[40m\x1b[31mr\x1b[32ma\x1b[33mi\x1b[34mn\x1b[35mb\x1b[36mo\x1b[37mw\x1b[0m"
 
@@ -191,9 +222,9 @@ class TestAnsi2HTML(unittest.TestCase):
             + '<span style="font-weight: bold; color: #F5F1DE; background-color: #000316">w</span>'
         )
 
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_produce_headers(self):
+    def test_produce_headers(self) -> None:
         conv = Ansi2HTMLConverter()
         headers = conv.produce_headers()
 
@@ -201,55 +232,55 @@ class TestAnsi2HTML(unittest.TestCase):
         with open(inputfile, "rb") as produce_headers:
             expected_data = read_to_unicode(produce_headers)
 
-        self.assertMultiLineEqual(headers, "".join(expected_data))
+        assert headers == "".join(expected_data)
 
-    def test_escaped_implicit(self):
+    def test_escaped_implicit(self) -> None:
         test = "<p>awesome</p>"
         expected = "&lt;p&gt;awesome&lt;/p&gt;"
         html = Ansi2HTMLConverter().convert(test, full=False)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_escaped_explicit(self):
+    def test_escaped_explicit(self) -> None:
         test = "<p>awesome</p>"
         expected = "&lt;p&gt;awesome&lt;/p&gt;"
         html = Ansi2HTMLConverter(escaped=True).convert(test, full=False)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_unescaped(self):
+    def test_unescaped(self) -> None:
         test = "<p>awesome</p>"
         expected = "<p>awesome</p>"
         html = Ansi2HTMLConverter(escaped=False).convert(test, full=False)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_markup_lines(self):
+    def test_markup_lines(self) -> None:
         test = "  wat  \n "
         expected = '<span id="line-0">  wat  </span>\n<span id="line-1"> </span>'
         html = Ansi2HTMLConverter(markup_lines=True).convert(test, full=False)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_no_markup_lines(self):
+    def test_no_markup_lines(self) -> None:
         test = "  wat  \n "
         expected = test
         html = Ansi2HTMLConverter().convert(test, full=False)
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_issue_25(self):
+    def test_issue_25(self) -> None:
         sample = "\x1b[0;38;5;238;48;5;231mTEXT\x1b[0m"
 
         html = Ansi2HTMLConverter(inline=False).convert(sample, full=False)
         expected = '<span class="ansi38-238 ansi48-231">TEXT</span>'
 
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_italic(self):
+    def test_italic(self) -> None:
         sample = "\x1b[3mITALIC\x1b[0m"
 
         html = Ansi2HTMLConverter(inline=True).convert(sample, full=False)
         expected = '<span style="font-style: italic">ITALIC</span>'
 
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_hidden_text(self):
+    def test_hidden_text(self) -> None:
         sample = "\x1b[%dmHIDDEN\x1b[%dmVISIBLE\x1b[0m" % (
             ANSI_VISIBILITY_OFF,
             ANSI_VISIBILITY_ON,
@@ -258,9 +289,9 @@ class TestAnsi2HTML(unittest.TestCase):
         html = Ansi2HTMLConverter(inline=True).convert(sample, full=False)
         expected = '<span style="visibility: hidden">HIDDEN</span>VISIBLE'
 
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_lighter_text(self):
+    def test_lighter_text(self) -> None:
         sample = "NORMAL\x1b[%dmLIGHTER\x1b[%dmBOLD\x1b[%dmNORMAL" % (
             ANSI_INTENSITY_REDUCED,
             ANSI_INTENSITY_INCREASED,
@@ -270,9 +301,9 @@ class TestAnsi2HTML(unittest.TestCase):
         html = Ansi2HTMLConverter(inline=True).convert(sample, full=False)
         expected = 'NORMAL<span style="font-weight: lighter">LIGHTER</span><span style="font-weight: bold">BOLD</span>NORMAL'
 
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_blinking_text(self):
+    def test_blinking_text(self) -> None:
         sample = "\x1b[%dm555\x1b[%dm666\x1b[%dmNOBLINK\x1b[0m" % (
             ANSI_BLINK_SLOW,
             ANSI_BLINK_FAST,
@@ -281,13 +312,13 @@ class TestAnsi2HTML(unittest.TestCase):
 
         html = Ansi2HTMLConverter(inline=True).convert(sample, full=False)
         expected = '<span style="text-decoration: blink">555</span><span style="text-decoration: blink">666</span>NOBLINK'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         html = Ansi2HTMLConverter(inline=False).convert(sample, full=False)
         expected = '<span class="ansi5">555</span><span class="ansi6">666</span>NOBLINK'
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_inverse_text(self):
+    def test_inverse_text(self) -> None:
         sample = "NORMAL\x1b[%dmINVERSE\x1b[%dmNORMAL\x1b[0m" % (
             ANSI_NEGATIVE_ON,
             ANSI_NEGATIVE_OFF,
@@ -296,7 +327,7 @@ class TestAnsi2HTML(unittest.TestCase):
         expected = (
             'NORMAL<span class="inv_background inv_foreground">INVERSE</span>NORMAL'
         )
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = "NORMAL\x1b[%dm303030\x1b[%dm!30!30!30\x1b[%dm303030\x1b[0m" % (
             30,
@@ -305,7 +336,7 @@ class TestAnsi2HTML(unittest.TestCase):
         )
         html = Ansi2HTMLConverter(inline=False).convert(sample, full=False)
         expected = 'NORMAL<span class="ansi30">303030</span><span class="inv30 inv_foreground">!30!30!30</span><span class="ansi30">303030</span>'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = (
             "NORMAL\x1b[%dm313131\x1b[%dm!31!31!31\x1b[%dm!31!43\x1b[%dm31+43\x1b[0mNORMAL"
@@ -313,87 +344,132 @@ class TestAnsi2HTML(unittest.TestCase):
         )
         html = Ansi2HTMLConverter(inline=False).convert(sample, full=False)
         expected = 'NORMAL<span class="ansi31">313131</span><span class="inv31 inv_foreground">!31!31!31</span><span class="inv31 inv43">!31!43</span><span class="ansi31 ansi43">31+43</span>NORMAL'
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_cross_line_state(self):  # covers issue 36, too
+    def test_cross_line_state(self) -> None:  # covers issue 36, too
         sample = "\x1b[31mRED\nSTILL RED"
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=False
         )
         expected = '<span style="color: #aa0000">RED\nSTILL RED</span>'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = "\x1b[31mRED\nSTILL RED\n"
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=False
         )
         expected = '<span style="color: #aa0000">RED\nSTILL RED\n</span>'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = "\x1b[31mRED\nSTILL RED"
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=True
         )
         expected = '<span style="color: #aa0000">RED\nSTILL RED</span>\n'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = "\x1b[31mRED\nSTILL RED\n"
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=True
         )
         expected = '<span style="color: #aa0000">RED\nSTILL RED\n</span>\n'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         sample = "\x1b[31mRED\nSTILL RED\x1b[m\n"
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=True
         )
         expected = '<span style="color: #aa0000">RED\nSTILL RED</span>\n'
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_scheme(self):  # covers issue 36, too
+    def test_scheme(self) -> None:  # covers issue 36, too
         sample = "\x1b[33mYELLOW/BROWN"
         # ansi2html scheme is brown #aa5500
         html = Ansi2HTMLConverter(inline=True).convert(
             sample, full=False, ensure_trailing_newline=False
         )
         expected = '<span style="color: #aa5500">YELLOW/BROWN</span>'
-        self.assertEqual(expected, html)
+        assert expected == html
 
         # xterm scheme is yellow #cdcd00
         html = Ansi2HTMLConverter(inline=True, scheme="xterm").convert(
             sample, full=False, ensure_trailing_newline=False
         )
         expected = '<span style="color: #cdcd00">YELLOW/BROWN</span>'
-        self.assertEqual(expected, html)
+        assert expected == html
 
-    def test_latex_inline(self):
+    def test_latex_inline(self) -> None:
         ansi = "\x1b[33mYELLOW/BROWN"
         target = "\\textcolor[HTML]{aa5500}{YELLOW/BROWN}"
         latex = Ansi2HTMLConverter(latex=True, inline=True).convert(ansi)
         assert target in latex
 
-    def test_latex_title(self):
+    def test_latex_title(self) -> None:
         ansi = ""
         title = "testing"
         target = "\\title{%s}" % title
         latex = Ansi2HTMLConverter(latex=True, inline=True, title=title).convert(ansi)
         assert target in latex
 
-    def test_latex_linkify(self):
+    def test_latex_linkify(self) -> None:
         ansi = "http://python.org/"
         target = "\\url{%s}" % ansi
         latex = Ansi2HTMLConverter(latex=True, inline=True, linkify=True).convert(ansi)
         assert target in latex
 
-    def test_command_script(self):
+    def test_truecolor(self) -> None:
+        ansi = (
+            "\u001b[38;2;255;102;102m 1 \u001b[0m "
+            + "\u001b[38;2;0;0;255m 2 \u001b[0m "
+            + "\u001b[48;2;65;105;225m 3 \u001b[0m"
+        )
+        target = (
+            '<span class="ansi38-255102102"> 1 </span> '
+            + '<span class="ansi38-000000255"> 2 </span> '
+            + '<span class="ansi48-065105225"> 3 </span>'
+        )
+        html = Ansi2HTMLConverter().convert(ansi)
+        assert target in html
+
+    def test_truecolor_inline(self) -> None:
+        ansi = (
+            "\u001b[38;2;255;102;102m 1 \u001b[0m "
+            + "\u001b[38;2;0;0;255m 2 \u001b[0m "
+            + "\u001b[48;2;65;105;225m 3 \u001b[0m"
+        )
+        target = (
+            '<span style="color: #FF6666"> 1 </span> '
+            + '<span style="color: #0000FF"> 2 </span> '
+            + '<span style="background-color: #4169E1"> 3 </span>'
+        )
+        html = Ansi2HTMLConverter(inline=True).convert(ansi)
+        assert target in html
+
+    def test_truecolor_malformed(self) -> None:
+        ansi = "\u001b[38;2;255;102m malformed \u001b[0m "
+        #                         ^ e.g. ";102" missed
+        target = '<span class="ansi2 ansi102"> malformed </span> '
+        html = Ansi2HTMLConverter().convert(ansi)
+        assert target in html
+
+    def test_256_color_malformed(self) -> None:
+        ansi = "\u001b[38;5m malformed \u001b[0m "
+        #                 ^ e.g. ";255" missed
+        target = '<span class="ansi5"> malformed </span> '
+        html = Ansi2HTMLConverter().convert(ansi)
+        assert target in html
+
+    def test_x_bit_color_malformed(self) -> None:
+        ansi = "\u001b[38m malformed \u001b[0m "
+        #               ^ e.g. ";5;255m" missed
+        target = '<span class="ansi38"> malformed </span> '
+        html = Ansi2HTMLConverter().convert(ansi)
+        assert target in html
+
+    def test_command_script(self) -> None:
         result = run(["ansi2html", "--version"], check=True)
         assert result.returncode == 0
 
-    def test_command_module(self):
+    def test_command_module(self) -> None:
         result = run(["python3", "-m", "ansi2html", "--version"], check=True)
         assert result.returncode == 0
-
-
-if __name__ == "__main__":
-    unittest.main()
